package jp.sourceforge.nicoro;

import android.app.Service;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;

// TODO 将来的にはFFmpegVideoDecoderやMediaPlayerをApplication経由で受け取って独自にバックグラウンド再生
// FIXME Service実行しっぱなしだと呼び出し元のActivityも死ななくなる？のでちょっとまずい。Activityが正常に終了しなかったエラーToastが表示される

public class BackgroundPlayerService extends Service
implements Handler.Callback {
    private static final String KEY_SAVE_CURRENT_PLAY_TIME = "key_save_current_play_time";
    private static final String KEY_SAVE_IS_PAUSE_PLAY = "key_save_is_pause_play";

    private static final int SAVE_TIMER_INTERVAL = 1000 * 5;

    private static final int MSG_ID_SAVE_PLAY_STATE = 0;

    private SharedPreferences.Editor mPrefEditorPlayState;
    private final HandlerWrapper mHandler = new HandlerWrapper(this);

    @Override
    public boolean handleMessage(Message msg) {
        switch (msg.what) {
            case MSG_ID_SAVE_PLAY_STATE:
                mPrefEditorPlayState.commit();
                break;
        }
        return true;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        mPrefEditorPlayState = NicoroApplication.getInstance(this).getPrefEditorPlayState();
    }

    @Override
    public void onStart(Intent intent, int startId) {
        handleStart(intent);
    }

    // API Level 5からのためoverrideは行わない
    public int onStartCommand(Intent intent, int flags, int startId) {
        handleStart(intent);
        return START_NOT_STICKY;
    }

    private void handleStart(Intent intent) {
        if (intent == null) {
            return;
        }
        if (intent.hasExtra(KEY_SAVE_CURRENT_PLAY_TIME)
                && intent.hasExtra(KEY_SAVE_IS_PAUSE_PLAY)) {
            Rational currentPlayTime = intent.getParcelableExtra(
                    KEY_SAVE_CURRENT_PLAY_TIME);
            AbstractPlayerFragment.putCurrentPlayTime(mPrefEditorPlayState,
                    currentPlayTime);
            boolean isPausePlay = intent.getBooleanExtra(KEY_SAVE_IS_PAUSE_PLAY, false);
            AbstractPlayerFragment.putIsPausePlay(mPrefEditorPlayState,
                    isPausePlay);
            mHandler.removeMessages(MSG_ID_SAVE_PLAY_STATE);
            mHandler.sendEmptyMessageDelayed(MSG_ID_SAVE_PLAY_STATE,
                    SAVE_TIMER_INTERVAL);
        }
    }

    @Override
    public void onDestroy() {
        mHandler.removeMessages(MSG_ID_SAVE_PLAY_STATE);
        mHandler.release();
        mPrefEditorPlayState.commit();
        super.onDestroy();
    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    public static ComponentName startServiceForPlayState(Context context,
            Rational currentPlayTime, boolean isPausePlay) {
        Intent intent = new Intent(context, BackgroundPlayerService.class)
            .putExtra(KEY_SAVE_CURRENT_PLAY_TIME, currentPlayTime)
            .putExtra(KEY_SAVE_IS_PAUSE_PLAY, isPausePlay);
        return context.startService(intent);
    }

    public static boolean stopService(Context context) {
        Intent intent = new Intent(context, BackgroundPlayerService.class);
        return context.stopService(intent);
    }
}
